@@ -91,8 +91,12 @@ class ODESys(object):
     append_iv :  bool
         See :attr:`append_iv`.
     autonomous_interface : bool (optional)
-        If given, sets the :attr:`autonomous` to indicate whether
+        If given, sets the :attr:`autonomous_interface` to indicate whether
         the system appears autonomous or not upon call to :meth:`integrate`.
+    autonomous_exprs : bool
+        Describes whether the independent variable appears in the rhs expressions.
+        If set to ``True`` the underlying solver is allowed to shift the
+        independent variable during integration.
 
     Attributes
     ----------
@@ -141,7 +145,8 @@ class ODESys(object):
                  band=None, names=(), param_names=(), indep_name=None, description=None, dep_by_name=False,
                  par_by_name=False, latex_names=(), latex_param_names=(), latex_indep_name=None,
                  taken_names=None, pre_processors=None, post_processors=None, append_iv=False,
-                 autonomous_interface=None, to_arrays_callbacks=None, **kwargs):
+                 autonomous_interface=None, to_arrays_callbacks=None, autonomous_exprs=None,
+                 _indep_autonomous_key=None, **kwargs):
         self.f_cb = _ensure_4args(f)
         self.j_cb = _ensure_4args(jac) if jac is not None else None
         self.dfdx_cb = dfdx
@@ -165,11 +170,12 @@ class ODESys(object):
         self.pre_processors = pre_processors or []
         self.post_processors = post_processors or []
         self.append_iv = append_iv
+        self.autonomous_exprs = autonomous_exprs
         if hasattr(self, 'autonomous_interface'):
             if autonomous_interface is not None and autonomous_interface != self.autonomous_interface:
                 raise ValueError("Got conflicting autonomous_interface infomation.")
         else:
-            if (autonomous_interface is None and getattr(self, 'autonomous_exprs', False) and
+            if (autonomous_interface is None and self.autonomous_exprs and
                len(self.post_processors) == 0 and len(self.pre_processors) == 0):
                 self.autonomous_interface = True
             else:
@@ -177,6 +183,7 @@ class ODESys(object):
 
         if self.autonomous_interface not in (True, False, None):
             raise ValueError("autonomous_interface needs to be a boolean value or None.")
+        self._indep_autonomous_key = _indep_autonomous_key
         self.to_arrays_callbacks = to_arrays_callbacks
         if len(kwargs) > 0:
             raise ValueError("Unknown kwargs: %s" % str(kwargs))
@@ -223,14 +230,24 @@ class ODESys(object):
         _names = [n for n in self.names if n not in self.taken_names]
         _y, tp_y = self._conditional_from_dict(y, self.dep_by_name, _names)
         _p, tp_p = self._conditional_from_dict(p, self.par_by_name, self.param_names)
+        del _names
 
         callbacks = callbacks or self.to_arrays_callbacks
         if callbacks is not None:  # e.g. dedimensionalisation
             if len(callbacks) != 3:
                 raise ValueError("Need 3 callbacks/None values.")
             _x, _y, _p = [e if cb is None else cb(e) for cb, e in zip(callbacks, [_x, _y, _p])]
+        _y = np.atleast_1d(_y)
+        if self._indep_autonomous_key:
+            if _y.shape[-1] == self.ny:
+                pass
+            elif _y.shape[-1] == self.ny - 1:
+                _y = np.concatenate((_y, _x[0]*np.ones(_y.shape[:-1] + (1,))), axis=-1)
+            else:
+                raise ValueError("y of incorrect shape")
 
-        arrs = [arr.T if tp else arr for tp, arr in zip([False, tp_y, tp_p], map(np.atleast_1d, (_x, _y, _p)))]
+        arrs = [arr.T if tp else arr for tp, arr in
+                zip([False, tp_y, tp_p], map(np.atleast_1d, (_x, _y, _p)))]
         extra_shape = None
         for a in arrs:
             if a.ndim == 1:
@@ -650,6 +667,7 @@ class ODESys(object):
                              " initialization instead)")
         if self.band is not None:
             kwargs['lband'], kwargs['uband'] = self.band
+        kwargs['autonomous_exprs'] = self.autonomous_exprs
         return self._integrate(pycvodes.integrate_adaptive,
                                pycvodes.integrate_predefined,
                                *args, **kwargs)
@@ -735,6 +753,11 @@ class ODESys(object):
                 np.abs(singular_values).min(axis=-1))
 
 
+class OdeSys(ODESys):
+    """ DEPRECATED, use ODESys instead. """
+    pass
+
+
 def _new_x(xout, x, guaranteed_autonomous):
     if guaranteed_autonomous:
         return 0, abs(x[-1] - xout[-1])  # rounding
@@ -742,7 +765,7 @@ def _new_x(xout, x, guaranteed_autonomous):
         return xout[-1], x[-1]
 
 
-def integrate_chained(odes, kw, x, y0, params=(), **kwargs):
+def integrate_auto_switch(odes, kw, x, y0, params=(), **kwargs):
     """ Auto-switching between formulations of ODE system.
 
     In case one has a formulation of a system of ODEs which is preferential in
@@ -836,6 +859,69 @@ def integrate_chained(odes, kw, x, y0, params=(), **kwargs):
         return Result(tot_x, tot_y, res.params, tot_nfo, odes[0])
 
 
-class OdeSys(ODESys):
-    """ DEPRECATED, use ODESys instead. """
-    pass
+integrate_chained = integrate_auto_switch  # deprecated name
+
+
+def chained_parameter_variation(subject, durations, y0, varied_params, default_params=None,
+                                integrate_kwargs=None, x0=None, npoints=1):
+    """ Integrate an ODE-system for a serie of durations with some parameters changed in-between
+
+    Parameters
+    ----------
+    subject : function or ODESys instance
+        If a function: should have the signature of :meth:`pyodesys.ODESys.integrate`
+        (and resturn a :class:`pyodesys.results.Result` object).
+        If a ODESys instance: the ``integrate`` method will be used.
+    durations : iterable of floats
+        Spans of the independent variable.
+    y0 : dict or array_like
+    varied_params : dict mapping parameter name (or index) to array_like
+        Each array_like need to be of same length as durations.
+    default_params : dict or array_like
+        Default values for the parameters of the ODE system.
+    integrate_kwargs : dict
+        Keyword arguments passed on to ``integrate``.
+    x0 : float-like
+        First value of independent variable. default: 0.
+    npoints : int
+        Number of points per sub-interval.
+
+    """
+    assert len(durations) > 0, 'need at least 1 duration (preferably many)'
+    for k, v in varied_params.items():
+        if len(v) != len(durations):
+            raise ValueError("Mismathced lengths of durations and varied_params")
+
+    if isinstance(subject, ODESys):
+        integrate = subject.integrate
+    else:
+        integrate = subject
+
+    default_params = default_params or {}
+    integrate_kwargs = integrate_kwargs or {}
+
+    def _get_idx(cont, idx):
+        if isinstance(cont, dict):
+            return {k: (v[idx] if hasattr(v, '__len__') and getattr(v, 'ndim', 1) > 0 else v)
+                    for k, v in cont.items()}
+        else:
+            return cont[idx]
+
+    durations = np.cumsum(durations)
+    for idx_dur in range(len(durations)):
+        params = default_params.copy()
+        for k, v in varied_params.items():
+            params[k] = v[idx_dur]
+        if idx_dur == 0:
+            if x0 is None:
+                x0 = durations[0]*0
+            out = integrate(np.linspace(x0, durations[0], npoints + 1), y0, params, **integrate_kwargs)
+        else:
+            if isinstance(out, Result):
+                out.extend_by_integration(durations[idx_dur], params, npoints=npoints, **integrate_kwargs)
+            else:
+                for idx_res, r in enumerate(out):
+                    r.extend_by_integration(durations[idx_dur], _get_idx(params, idx_res),
+                                            npoints=npoints, **integrate_kwargs)
+
+    return out
