@@ -16,7 +16,7 @@ import numpy as np
 from .util import import_
 from .core import ODESys, RecoverableError
 from .util import (
-    transform_exprs_dep, transform_exprs_indep, _ensure_4args, _Wrapper
+    transform_exprs_dep, transform_exprs_indep, _ensure_4args, _Callback
 )
 
 Backend = import_('sym', 'Backend')
@@ -77,6 +77,24 @@ def _get_lin_invar_mtx(lin_invar, be, ny, names=None):
         return li_mtx
 
 
+def _is_autonomous(indep, exprs):
+    """ Whether the expressions for the dependent variables are autonomous.
+
+    Note that the system may still behave as an autonomous system on the interface
+    of :meth:`integrate` due to use of pre-/post-processors.
+    """
+    if indep is None:
+        return True
+    for expr in exprs:
+        try:
+            in_there = indep in expr.free_symbols
+        except:
+            in_there = expr.has(indep)
+        if in_there:
+            return False
+    return True
+
+
 class SymbolicSys(ODESys):
     """ ODE System from symbolic expressions
 
@@ -147,14 +165,12 @@ class SymbolicSys(ODESys):
         Closed form expression for how to compute the first step.
     roots : iterable of expressions or None
         Roots to report for during integration.
-    be : module
-        Symbolic backend, e.g. ``sympy`` or ``symcxx``.
+    backend : module
+        Symbolic backend, e.g. 'sympy', 'symengine'.
     ny : int
         ``len(self.dep)`` note that this is not neccessarily the expected length of
         ``y0`` in the case of e.g. :class:`PartiallySolvedSystem`. i.e. ``ny`` refers
         to the number of dependent variables after pre processors have been applied.
-    be : module
-        Symbolic backend.
     init_indep : Symbol,  ``None``
         Symbol for initial value of independent variable (before pre processors).
     init_dep : tuple of Symbols or ``None``
@@ -164,7 +180,8 @@ class SymbolicSys(ODESys):
 
     _attrs_to_copy = ('first_step_expr', 'names', 'param_names', 'dep_by_name', 'par_by_name',
                       'latex_names', 'latex_param_names', 'description', 'linear_invariants',
-                      'linear_invariant_names', 'to_arrays_callbacks')
+                      'linear_invariant_names', 'to_arrays_callbacks', '_indep_autonomous_key',
+                      'taken_names')
     append_iv = True
 
     @property
@@ -189,7 +206,7 @@ class SymbolicSys(ODESys):
                 raise ValueError("Pass params explicitly or pass True to have them deduced.")
             params = all_free
 
-        self.params = params
+        self.params = tuple(params)
         self._jac = jac
         self._dfdx = dfdx
         self.first_step_expr = first_step_expr
@@ -228,8 +245,9 @@ class SymbolicSys(ODESys):
             kwargs['param_names'] = [p.name for p in self.params]
 
         self.band = kwargs.get('band', None)  # needed by get_j_ty_callback
-        self.lower_bounds = None if lower_bounds is None else np.array(lower_bounds)  # needed by get_f_ty_callback
-        self.upper_bounds = None if upper_bounds is None else np.array(upper_bounds)  # needed by get_f_ty_callback
+        # bounds needed by get_f_ty_callback:
+        self.lower_bounds = None if lower_bounds is None else np.array(lower_bounds)*np.ones(self.ny)
+        self.upper_bounds = None if upper_bounds is None else np.array(upper_bounds)*np.ones(self.ny)
 
         super(SymbolicSys, self).__init__(
             self.get_f_ty_callback(),
@@ -238,6 +256,7 @@ class SymbolicSys(ODESys):
             self.get_first_step_callback(),
             self.get_roots_callback(),
             nroots=None if roots is None else len(roots),
+            autonomous_exprs=_is_autonomous(self.indep, self.exprs),
             **kwargs)
 
         self.linear_invariants = linear_invariants
@@ -434,35 +453,13 @@ class SymbolicSys(ODESys):
         """ Number of dependent variables in the system. """
         return len(self.exprs)
 
-    @property
-    def autonomous_exprs(self):
-        """ Whether the expressions for the dependent variables are autonomous.
-
-        Note that the system may still behave as an autonomous system on the interface
-        of :meth:`integrate` due to use of pre-/post-processors.
-        """
-        if hasattr(self, '_autonomous_exprs'):
-            return self._autonomous_exprs
-        if self.indep is None:
-            self._autonomous_exprs = True
-            return True
-        for expr in self.exprs:
-            try:
-                in_there = self.indep in expr.free_symbols
-            except:
-                in_there = expr.has(self.indep)
-            if in_there:
-                self._autonomous_exprs = False
-                return False
-        self._autonomous_exprs = True
-        return True
-
-    def as_autonomous(self, indep_name=None, latex_indep_name=None):
+    def as_autonomous(self, new_indep_name=None, new_latex_indep_name=None):
         if self.autonomous_exprs:
             return self
-        new_names = () if not self.names else (self.names + (self.indep_name,))
-        new_indep_name = indep_name or _get_indep_name(new_names)
-        new_latex_indep_name = latex_indep_name or new_indep_name
+        old_indep_name = self.indep_name or _get_indep_name(self.names)
+        new_names = () if not self.names else (self.names + (old_indep_name,))
+        new_indep_name = new_indep_name or _get_indep_name(new_names)
+        new_latex_indep_name = new_latex_indep_name
         new_latex_names = () if not self.latex_names else (
             self.latex_names + (new_latex_indep_name,))
         new_indep = self.be.Symbol(new_indep_name)
@@ -476,24 +473,13 @@ class SymbolicSys(ODESys):
             autonomous_interface=False,  # see pre-processor below
         )
         if new_names:
-            new_kw['taken_names'] = self.taken_names + (self.indep_name,)
+            new_kw['taken_names'] = self.taken_names + (old_indep_name,)
         if self.linear_invariants:
             new_kw['linear_invariants'] = np.concatenate(
                 (self.linear_invariants, np.zeros((self.linear_invariants.shape[0], 1))), axis=-1)
         for attr in filter(lambda k: k not in new_kw, self._attrs_to_copy):
             new_kw[attr] = getattr(self, attr)
 
-        def autonomous_pre_processor(x, y, p):
-            if isinstance(y, dict) and not self.dep_by_name:
-                y = [y[k] for k in self._ori_sys.dep]
-            if isinstance(p, dict) and not self.par_by_name:
-                p = [p[k] for k in self._ori_sys.params]
-            x, y, p = map(np.atleast_1d, (x, y, p))
-            if y.ndim == 2:
-                return zip(*[autonomous_pre_processor(_x, _y, _p)
-                             for _x, _y, _p in zip(x, y, p)])
-            return x, _append(y, [x[0]]), p
-
         def autonomous_post_processor(x, y, p):
             try:
                 y[0][0, 0]
@@ -505,8 +491,9 @@ class SymbolicSys(ODESys):
             # one could check here that x and y[..., -1] do not differ too much
             return y[..., -1], y[..., :-1], p
 
-        new_kw['pre_processors'] = self.pre_processors + [autonomous_pre_processor]
         new_kw['post_processors'] = [autonomous_post_processor] + self.post_processors
+
+        new_kw['_indep_autonomous_key'] = new_names[-1] if new_names else True
         return self.__class__(zip(new_dep, new_exprs), indep=new_indep, params=self.params, **new_kw)
 
     def get_jac(self):
@@ -543,9 +530,7 @@ class SymbolicSys(ODESys):
         return self._dfdx
 
     def _callback_factory(self, exprs):
-        args = [self.indep] + list(self.dep) + list(self.params)
-        return _Wrapper(self.be.Lambdify(args, exprs),
-                        len(self.dep), len(self.params))
+        return _Callback(self.indep, self.dep, self.params, exprs, self.be.Lambdify)
 
     def get_f_ty_callback(self):
         """ Generates a callback for evaluating ``self.exprs``. """
@@ -600,31 +585,6 @@ class SymbolicSys(ODESys):
             return None
         return self._callback_factory(invar)
 
-    # Not working yet:
-    def _integrate_mpmath(self, xout, y0, params=()):
-        """ Not working at the moment, need to fix
-        (low priority - taylor series is a poor method)"""
-        raise NotImplementedError
-        xout, y0, self.internal_params = self.pre_process(xout, y0, params)
-        from mpmath import odefun
-
-        def rhs(x, y):
-            rhs.ncall += 1
-            return [
-                e.subs(
-                    ([(self.indep, x)] if self.indep is not None else []) +
-                    list(zip(self.dep, y))
-                ) for e in self.exprs
-            ]
-        rhs.ncall = 0
-
-        cb = odefun(rhs, xout[0], y0)
-        yout = []
-        for x in xout:
-            yout.append(cb(x))
-        info = {'nrhs': rhs.ncall}
-        return self.post_process(xout, yout, self.internal_params)[:2] + (info,)
-
     def _get_analytic_stiffness_cb(self):
         J = self.get_jac()
         eig_vals = list(J.eigenvals().keys())
@@ -727,7 +687,11 @@ class TransformedSys(SymbolicSys):
                                           list(kwargs.get('nonlinear_invariant_names') or ())))
 
         lower_b = kwargs.pop('lower_bounds', None)
+        if lower_b is not None:
+            lower_b *= np.ones(len(dep))
         upper_b = kwargs.pop('upper_bounds', None)
+        if upper_b is not None:
+            upper_b *= np.ones(len(dep))
         pre_processors = kwargs.pop('pre_processors', [])
         post_processors = kwargs.pop('post_processors', [])
         super(TransformedSys, self).__init__(
@@ -805,20 +769,16 @@ class TransformedSys(SymbolicSys):
         else:
             return zip(*[self._back_transform_out(_x, _y, _p) for
                          _x, _y, _p in zip(xout, yout, params)])
-        xout, yout, params = map(np.asarray, (xout, yout, params))
-        xbt = np.empty(xout.size)
-        ybt = np.empty((xout.size, self.ny))
-        for idx, (x, y) in enumerate(zip(xout.flat, yout)):
-            xbt[idx] = x if self.b_indep is None else self.b_indep(x, y, params)
-            ybt[idx, :] = y if self.b_dep is None else self.b_dep(x, y, params)
-        return xbt, ybt, params
+        x = xout if self.b_indep is None else self.b_indep(xout, yout, params).squeeze(axis=-1)
+        y = yout if self.b_dep is None else self.b_dep(xout, yout, params)
+        return x, y, params
 
     def _forward_transform_xy(self, x, y, p):
         x, y, p = map(np.asarray, (x, y, p))
         if y.ndim == 1:
-            return (x if self.f_indep is None else
-                    np.concatenate([self.f_indep(_, y, p) for _ in x]),
-                    y if self.f_dep is None else self.f_dep(x[0], y, p), p)
+            _x = x if self.f_indep is None else self.f_indep(x, y[..., None, :], p[..., None, :])[..., 0]
+            _y = y if self.f_dep is None else self.f_dep(x[..., 0], y, p)
+            return _x, _y, p
         elif y.ndim == 2:
             return zip(*[self._forward_transform_xy(_x, _y, _p) for _x, _y, _p in zip(x, y, p)])
         else:
@@ -1002,8 +962,8 @@ def _append(arr, *iterables):
     return arr
 
 
-def _concat(*args):
-    return np.concatenate(list(map(np.atleast_1d, args)))
+# def _concat(*args):
+#     return np.concatenate(list(map(np.atleast_1d, args)))
 
 
 class PartiallySolvedSystem(SymbolicSys):
@@ -1211,8 +1171,8 @@ class PartiallySolvedSystem(SymbolicSys):
 
     @staticmethod
     def _get_analytic_callback(ori_sys, analytic_exprs, new_dep, new_params):
-        args = _concat(ori_sys.indep, new_dep, new_params)
-        return _Wrapper(ori_sys.be.Lambdify(args, analytic_exprs), len(new_dep), len(new_params))
+        # args = _concat()
+        return _Callback(ori_sys.indep, new_dep, new_params, analytic_exprs, ori_sys.be.Lambdify)
 
     def __getitem__(self, key):
         ori_dep = self.original_dep[self.names.index(key)]
