@@ -3,11 +3,13 @@ from __future__ import (absolute_import, division, print_function)
 
 from ..util import requires
 
+import pytest
 
 from pyodesys import ODESys
 from .test_core import vdp_f, vdp_j
 
 
+@pytest.mark.slow
 @requires('matplotlib')
 def test_plot_result():
     odes = ODESys(vdp_f, vdp_j)
